syntax = "proto2";
package buffers.resources;

import "options.proto";

message Room {
  optional int32 id = 29 [(gmx) = "GMX_DEPRECATED"];

  optional string caption = 1;
  optional uint32 width = 2 [(yyp) = "roomSettings/Width"];
  optional uint32 height = 3 [(yyp) = "roomSettings/Height"];
  optional uint32 hsnap = 4;
  optional uint32 vsnap = 5;
  optional bool isometric = 6;
  optional uint32 speed = 27;
  optional bool persistent = 7 [(yyp) = "roomSettings/persistent"];
  optional int32 color = 8 [(gmx) = "colour"];
  optional bool show_color = 9 [(gmx) = "showcolour"];
  optional string creation_code = 10;
  optional bool enable_views = 11 [(gmx) = "enableViews", (yyp) = "viewSettings/enableViews"];
  optional bool clear_view_background = 12 [(gmx) = "clearViewBackground", (yyp) = "viewSettings/clearViewBackground"];
  optional bool clear_display_buffer = 13 [(gmx) = "clearDisplayBuffer", (yyp) = "viewSettings/clearDisplayBuffer"];

  message Background {
    optional bool visible = 1;
    optional bool foreground = 2;
    optional string background_name = 3 [(resource_ref)="background", (gmx) = "name"];
    optional int32 x = 4;
    optional int32 y = 5;
    optional bool htiled = 6;
    optional bool vtiled = 7;
    optional double hspeed = 8;
    optional double vspeed = 9;
    optional bool stretch = 10;
    optional int32 color = 11 [default = 0xFFFFFF];
    optional double alpha = 12 [default = 1.0];
  }

  repeated Background backgrounds = 15 [(gmx) = "backgrounds/background"];

  message View {
    optional bool visible = 1;
    optional int32 xview = 2;
    optional int32 yview = 3;
    optional uint32 wview = 4;
    optional uint32 hview = 5;
    optional int32 xport = 6;
    optional int32 yport = 7;
    optional uint32 wport = 8;
    optional uint32 hport = 9;
    optional int32 hborder = 10;
    optional int32 vborder = 11;
    optional double hspeed = 12;
    optional double vspeed = 13;
    optional string object_following = 14 [(resource_ref)="object", (gmx) = "objName"];
  }

  repeated View views = 16 [(gmx) = "views/view", (yyp) = "views"];

  message Instance {
    optional string name = 1;
    optional int32 id = 2 [(id_start) = 100001];
    optional string object_type = 3 [(resource_ref)="object", (gmx) = "objName"];
    optional double x = 4;
    optional double y = 5;
    optional double z = 6;
    optional string creation_code = 7;
    optional string initialization_code = 14;
    optional double xscale = 8 [default = 1, (gmx) = "scaleX"];
    optional double yscale = 9 [default = 1, (gmx) = "scaleY"];
    optional double zscale = 10 [default = 1, (gmx) = "scaleY"];
    optional double rotation  = 11;
    optional double zrotation = 12;
    optional int32 color = 13 [default = 0xFFFFFF, (gmx) = "colour"];

    message EditorSettings {
      optional bool locked = 1;
    }

    optional EditorSettings editor_settings = 31;
  }

  repeated Instance instances = 17 [(gmx) = "instances/instance"];

  message Tile {
    optional string name = 1;
    optional int32 id = 2 [(id_start) = 10000001];
    optional string background_name = 3 [(resource_ref)="background", (gmx) = "bgName"];
    optional double x = 4;
    optional double y = 5;
    optional int32 xoffset = 6 [(gmx) = "xo"];
    optional int32 yoffset = 7 [(gmx) = "yo"];
    optional uint32 width = 8 [(gmx) = "w"];
    optional uint32 height = 9 [(gmx) = "h"];
    optional int32 depth = 10;
    optional double xscale = 11 [default = 1.0, (gmx) = "scaleX"];
    optional double yscale = 12 [default = 1.0, (gmx) = "scaleY"];
    optional int32 color = 13 [default = 0xFFFFFF, (gmx) = "colour"];
    optional double alpha = 14 [default = 1.0];

    message EditorSettings {
      optional bool locked = 1;
    }

    optional EditorSettings editor_settings = 31;
  }

  repeated Tile tiles = 18 [(gmx) = "tiles/tile"];

  message Layer {
    optional string name = 1;
    optional int32 id = 2;
    optional int32 depth = 3;
    optional bool visible = 4;

    repeated Instance instances = 5;
    repeated Tile tiles = 6;
  }

  repeated Layer layers = 20;

  message RoomPhysicsSettings {
    optional bool use_physics = 1 [(gmx) = "PhysicsWorld"];
    optional int32 phy_world_top = 2 [(gmx) = "PhysicsWorldTop"];
    optional int32 phy_world_left = 3 [(gmx) = "PhysicsWorldLeft"];
    optional int32 phy_world_right = 4 [(gmx) = "PhysicsWorldRight"];
    optional int32 phy_world_bottom = 5 [(gmx) = "PhysicsWorldBottom"];
    optional double phy_world_xgravity = 6 [(gmx) = "PhysicsWorldGravityX"];
    optional double phy_world_ygravity = 7 [(gmx) = "PhysicsWorldGravityY"];
    optional double phy_world_pixel_meters = 8 [(gmx) = "PhysicsWorldPixToMeters"];
  }

  optional RoomPhysicsSettings physics_settings = 19 [(gmx) = "EGM_NESTED"];

}
